# Copyright (c) 2021-2023 Alibaba Cloud Computing Ltd.
# SPDX-License-Identifier: MulanPSL-2.0
import os
import sys
import logging

from logging import config as logconf
from configparser import ConfigParser

LOG_CONF = {
    "version": 1,
    "formatters": {
        "std": {
            "format": "PID=%(process)d %(asctime)s %(levelname)s : %(message)s - %(filename)s:%(lineno)d"
        },
        "simple": {
            "format": "%(asctime)s %(levelname)s : %(message)s"
        }
    },
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
            "level": "WARNING",
            "formatter": "simple",
            "stream": sys.stdout
        },
        "logfile": {
            "class": "logging.handlers.TimedRotatingFileHandler",
            "filename": "/var/log/keentune/target.log",
            "interval": 1,
            "backupCount": 14,
            "level": "INFO",
            "formatter": "std",
        }
    },
    "loggers": {
        "common": {
            "level": "INFO",
            "handlers": ["console","logfile"],
            "propagate": False
        },
        "auto": {
            "level": "DEBUG",
            "handlers": ["logfile"],
            "propagate": False
        }
    },
    "root": {
        "level": "DEBUG",
        "handlers": ["console"]
    }
}

class Config:
    conf_file_path = "/etc/keentune/target/keenopt.conf"
    conf = ConfigParser()
    conf.read(conf_file_path)

    LOG_CONF['handlers']['console']['level']       = conf['log']['CONSOLE_LEVEL']
    LOG_CONF['handlers']['logfile']['level']       = conf['log']['LOGFILE_LEVEL']

    LOG_CONF['handlers']['logfile']['filename']    = conf['log']['LOGFILE_PATH']
    LOG_CONF['handlers']['logfile']['interval']    = int(conf['log']['LOGFILE_INTERVAL'])
    LOG_CONF['handlers']['logfile']['backupCount'] = int(conf['log']['LOGFILE_BACKUP_COUNT'])

    logconf.dictConfig(LOG_CONF)
    logger = logging.getLogger('common')


