# Copyright (c) 2021-2023 Alibaba Cloud Computing Ltd.
# SPDX-License-Identifier: MulanPSL-2.0
import math  
import numpy as np  

from keenopt.common.log import funcLog
from keenopt.strategy.DFSGradientDescent import DFSGradientDescent

class BFSNode:
    def __init__(self, index, perturbation, loss_value):
        super().__init__()
        self.index = index
        self.perturbation = perturbation
        self.loss_value = loss_value
        self.child = []

class BFSGradientDescent(DFSGradientDescent):
    @funcLog
    def setOrigin(self, 
                   x: np.array, 
                   fx: np.array, 
                   enable_index: list):

        fx_weighted = np.sum(fx * self.fx_weight,axis=1)
        
        if self.Stack.__len__() == 0:
            sorted_index = np.argsort(np.squeeze(fx_weighted)).tolist()
            
            for index in sorted_index:
                self.Stack.append(
                    BFSNode(index = index, perturbation = self.init_perturbation,loss_value = fx_weighted[index]))
            
            self.batch_node = [self.Stack.pop() for _ in range(self.batch_size)]

        else:
            for i in enable_index:
                # Case 1. Find better point
                if fx_weighted[self.batch_node[i].index] > fx_weighted[len(fx_weighted) - self.batch_size + i]:
                    self.batch_node[i].child.append(BFSNode(
                        index = len(fx_weighted) - self.batch_size + i, 
                        perturbation = self.batch_node[i].perturbation,
                        loss_value = fx_weighted[len(fx_weighted) - self.batch_size + i]
                    ))
                
                # Case 2. Fail to find better point
                else:
                    # Case 2.1. Replace seed
                    if self.batch_node[i].perturbation <= self.min_perturbation:
                        if self.batch_node[i].child.__len__() == 0:
                            self.batch_node[i] = self.Stack.pop()
                        else:
                            self.batch_node[i].child = sorted(self.batch_node[i].child, key = lambda i:i.loss_value)
                            self.batch_node[i] = self.batch_node[i].child[0]
                            
                    # Case 2.3 Continue search, reduce perturbation
                    else:
                        self.batch_node[i].perturbation *= self.perturbation_reduce_rate
                
        best_index = np.array([self.batch_node[i].index for i in range(self.batch_size)])
        return x[best_index,...], fx[best_index,...]
