# Copyright (c) 2021-2023 Alibaba Cloud Computing Ltd.
# SPDX-License-Identifier: MulanPSL-2.0
import torch
import torch.nn as nn
import math

class CNNregression(nn.Module):
    def __init__(self,x_dim,fx_dim):
        super(CNNregression, self).__init__()
        self.x_dim = x_dim
        self.fx_dim = fx_dim
        self.in_features = math.floor(self.x_dim/2-1)*32
        self.conv = nn.Sequential(
            nn.Conv1d(kernel_size=3, in_channels=6, out_channels=8, stride=1, padding=1), 
            nn.SELU(),
            nn.MaxPool1d(kernel_size=2, stride=1), 
            nn.Conv1d(kernel_size=3, in_channels=8, out_channels=8*2, stride=1, padding=1),
            nn.SELU(),
            nn.MaxPool1d(kernel_size=2, stride=1), 
            nn.Conv1d(kernel_size=3, in_channels=8*2, out_channels=8*4, stride=1, padding=1), 
            nn.SELU(),
            nn.MaxPool1d(kernel_size=2, stride=2)
        )
      
        self.unique_layer = []
        for _ in range(self.fx_dim):
            self.unique_layer.append(nn.Sequential(
            nn.Linear(in_features=self.in_features, out_features=512),
            nn.Dropout(p=0.2),
            nn.Linear(in_features=512, out_features=1)
        ),
            )
    def forward(self, x):
        x = torch.tensor(x)
        x = x.permute(0, 2, 1)
        x = self.conv(x)
        common_feature = x.view(x.size(0), -1)
        output = torch.empty(common_feature.shape[0],0)
        for i in range(self.fx_dim):
            _out = self.unique_layer[i](common_feature)
            output = torch.cat((output, _out), 1)
        return output

